#!/bin/bash
#set -n		# read but do not execute commands (check on syntax errors)
#
# File: cgi-bin/count.sh
#
# Implements a counter for websites.
# Inc. a counter, store the new value and produce a png from that number.
#
#    Copyright (C) 2008  Juergen Kaesmann (JK)

##############################################################################
# GPL NOTICE

#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program; if not, write to the Free Software
#    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

##############################################################################
# EXIT CODES
# Standard

##############################################################################
# SOURCE FILES

#. /usr/local/etc/main.conf				# config main
#. $LLIBDIR/libfuncstd-sh				# functions standard
. /usr/local/etc/counter.conf				# config counter
#. ~/.samplerc						# config user

##############################################################################
# DECLARATION OF CONSTANTS AND VARIABLES

#------------------STANDARD CONSTANTS AND VARIABLES---------------------------
readonly package=Counter-Server				# package name
readonly vinfopath='/usr/local/share/ksm-counter-server' # path to package dir
readonly version=$(/bin/cat $vinfopath/VERSION)		# package version
readonly release=$(/bin/cat $vinfopath/RELEASE)		# package release
readonly scriptname=$(basename $0)			# name of script
readonly tempfile=$TEMPDIR/$scriptname.$UID.tmp		# temporary file name
#tempfile=$(mktemp $TEMPDIR/$scriptname.$UID.XXXXXX)	# temporary file name
readonly debuglog="/bin/logger -p user.debug -t $scriptname -- DEBUG:"	# log dbg msg
readonly errlog="/bin/logger -p user.err -t $scriptname -- ERROR:"	# log err msg
readonly infolog="/bin/logger -p user.info -t $scriptname -- INFO:"	# log inf msg
cmd=''							# sub command
debug=no						# debug=yes/no
verbose=''						# verbose=yes/''/no

#------------------USER FILL IN AFTER INSTALLATION----------------------------
#echo "$scriptname line 59: Setup following 2 vars. Then comment out this line." >&2; exit 1
#readonly var1='Dummy1'
#var2='Dummy2'

#-----------------------USER-CONSTANTS----------------------------------------
# STRINGS
#readonly const=string

# NUMBERS
#readonly const=value

#-----------------------USER-VARIABLES----------------------------------------
# STRINGS
# WebDir=htdocs (from config)
partsdir=$vinfopath/counter-parts			# parts dir
datadir=../counter-dat					# data dir
cntrdir=../$WebDir/counter				# dir of counter img
prefn='cnt-'						# img filename prefix
prepn="$partsdir/$prefn"				# img templ path pref.
pwdfn='/usr/local/etc/counter.pwd'			# password file

#args=''				# all args from cmdline
arg1=''					# 1. arg from cmdline,base fn(counter)
arg2=''					# 2. arg from cmdline, user cmd
arg4=''					# 4. arg from cmdline, password
#opt_i=''				# option -i (arg)
#opt_o=''				# option -o (yes)
#sample-opt=''				# option --sample-opt (yes/no/arg)
#infile=''				# option --infile (arg)
#outfile=''				# option --outfile (arg)
datfn=''				# cntr data file
imgfn=''				# cntr img file
z1=''					# str with sngl number
z2=''					# str with sngl number
z3=''					# str with sngl number
z4=''					# str with sngl number
z5=''					# str with sngl number
z6=''					# str with sngl number
cntstr=''				# final images string

# NUMBERS
err=0					# error code
arg3=0					# 3.arg from cmdline,cntr val to store
nr=0					# number to process

# ARRAYS
# Indexed arrays are always zero based string arrays.
# If index starts with 1 and all members are assigned then
# count of members of array = last index of array
#declare -a arr
#maxarr="${#arr[@]}"			# get number of elements in arr
#unset arr[@]				# remove the entire array

##############################################################################
# SIGNAL HANDLING

##############################################################################
# FUNCTIONS

JKK
#=============================================================================
function helptext ()
{
# Output helptext
# Uses cat, echo, logger
# Return codes:
# Standard

if [ "$debug" == "yes" ]; then				# DEBUG
	echo "DEBUG helptext: $# $@" >&2
	$debuglog helptext: $# $@
fi

cat <<EOT
NAME
$scriptname - implement a counter for webpages

SYNOPSIS
$scriptname [command] [options] [file [cmd nr pwd]]

DESCRIPTION
This is step 2 of the webcounter package. It gets the old counter value, 
inc. this counter, stores the new value and produces a png (100 x 30 px) from 
that number. The counter is in the range from 0 to 999999.

Uses 2 borders l/r and 10 digits:
${prefn}l.pnm, ${prefn}r.pnm, ${prefn}0.pnm..${prefn}9.pnm (all preformatted).
The number is stored in $datadir/
The png is stored in $cntrdir/

The counter cmd 'set' sets the counter to the specified value 'nr'.

COMMANDS
Give only one or none:
  -h|--help    = Output help, then exit.
  -V|--version = Output version info, then exit.

OPTIONS
Gnu style options:

  -d|--debug   = Turn on DEBUG mode.
  -q|--quiet   = Be quiet.
  -v|--verbose = Be verbose.
  --           = End of options.

ARGUMENTS
Give if needed:

  file = File to store counter value
  cmd  = Command for the counter: set
  nr   = Value for cmd 'set': 0-999999
  pwd  = Password for this counter

EXIT STATUS
On program exit:

  Code 0 = OK
  Code 2 = Wrong parameter count
  Code 3 = Unspecified error
  Code 4 = Unknown option

  See output of 'showerrmsg'.

FILES
Used directories and files:

  Config Global = /usr/local/etc/sample.conf
  Config User   = ~/.samplerc

SEE ALSO
Related manpages:

  showerrmsg(1).

AUTHORS
Juergen Kaesmann <juergen@kaesmann-online.de>
EOT
#  -o|--output  = Output goes to STDOUT.
#  -i f         = Input from file <f>.
#  --infile=f   = Input from file <f>.

#EXAMPLES
#For instance:
#
#  Do it:
#  $scriptname

return
}

jkk
#=============================================================================
function get_ziffer_image ()
{
# add a ziffer image to cntstr
# par1 = ziffer
# uses logger
# Return codes:
# Standard

if [ "$debug" == "yes" ]; then				# DEBUG
	#echo "DEBUG get_ziffer_image: $# $@" >&2
	$debuglog get_ziffer_image: $# $@
fi

local p1="$1" ret=0

case "$p1" in
    0) cntstr="${cntstr}${prepn}0.pnm ";;
    1) cntstr="${cntstr}${prepn}1.pnm ";;
    2) cntstr="${cntstr}${prepn}2.pnm ";;
    3) cntstr="${cntstr}${prepn}3.pnm ";;
    4) cntstr="${cntstr}${prepn}4.pnm ";;
    5) cntstr="${cntstr}${prepn}5.pnm ";;
    6) cntstr="${cntstr}${prepn}6.pnm ";;
    7) cntstr="${cntstr}${prepn}7.pnm ";;
    8) cntstr="${cntstr}${prepn}8.pnm ";;
    9) cntstr="${cntstr}${prepn}9.pnm ";;
    *) ret=1;;					# unknown value
esac

return $ret
}

#=============================================================================
function make_image ()
{
# make a counter image and store it
# uses logger, pnmcat, pnmtopng
# Return codes:
# Standard

if [ "$debug" == "yes" ]; then				# DEBUG
	#echo "DEBUG make_image: $# $@" >&2
	$debuglog make_image: $# $@
fi

local ret=0

z1=${nr:0:1}						# separate digit 1
z2=${nr:1:1}						# separate digit 2
z3=${nr:2:1}						# separate digit 3
z4=${nr:3:1}						# separate digit 4
z5=${nr:4:1}						# separate digit 5
z6=${nr:5:1}						# separate digit 6

cntstr="${prepn}l.pnm "					# get left border
get_ziffer_image $z1					# add digit image 1
ret=$?
if [ "$ret" -eq 0 ]; then
	get_ziffer_image $z2				# add digit image 2
	ret=$?
fi
if [ "$ret" -eq 0 ]; then
	get_ziffer_image $z3				# add digit image 3
	ret=$?
fi
if [ "$ret" -eq 0 ]; then
	get_ziffer_image $z4				# add digit image 4
	ret=$?
fi
if [ "$ret" -eq 0 ]; then
	get_ziffer_image $z5				# add digit image 5
	ret=$?
fi
if [ "$ret" -eq 0 ]; then
	get_ziffer_image $z6				# add digit image 6
	ret=$?
fi
if [ "$ret" -eq 0 ]; then
	cntstr="${cntstr}${prepn}r.pnm"			# add right border
	pnmcat -lr $cntstr | pnmtopng -quiet > $imgfn	# make cntr img
fi

return $ret
}

#=============================================================================
function make_inc_image ()
{
# increment the counter, make a counter image and store it
# uses echo, logger
# Return codes:
# Standard

if [ "$debug" == "yes" ]; then				# DEBUG
	#echo "DEBUG make_inc_image: $# $@" >&2
	$debuglog make_inc_image: $# $@
fi

local ret=0

let nr++						# inc counter
if [ "$nr" -gt 999999 ]; then				# if overflow then
	nr=0						# set to 0
fi
echo "$nr" > $datfn					# save counter
while [ "${#nr}" -lt 6 ]; do				# leading digs = 0
	nr=0$nr
done

make_image

return $ret
}

#=============================================================================
function check_pswd ()
{
# check the password of an counter
# par1 = counter
# par2 = password
# uses cat, cut, echo, grep, logger
# Return codes:
# Standard

if [ "$debug" == "yes" ]; then				# DEBUG
	#echo "DEBUG check_pswd: $# $@" >&2
	$debuglog check_pswd: $# $@
fi

local cntr="$1" pswd="$2" s='' ss=''
local ret=0

s=`cat $pwdfn | grep -m 1 -s "^[[:space:]]*${cntr};"`
ret=$?
if [ "$ret" -eq 0 ]; then
	ss=`echo "$s" | cut -d ';' -f 2`
	ret=$?
	if [ "$ret" -eq 0 ]; then
		if [ "$ss" != "$pswd" ]; then
			ret=3
		fi
	fi
fi

return $ret
}

#=============================================================================
function set_counter ()
{
# set the counter to a specific value
# uses echo, logger
# Return codes:
# Standard

if [ "$debug" == "yes" ]; then				# DEBUG
	#echo "DEBUG set_counter: $# $@" >&2
	$debuglog set_counter: $# $@
fi

local pswd=$arg4
local cval=$arg3 ret=0

check_pswd "$arg1" "$pswd"				# check password
ret=$?
if [ "$ret" -eq 0 ]; then				# password ok
	echo "$cval" > $datfn
fi

return $ret
}

##############################################################################
# PARSE COMMANDLINE
# Try to read all options with args and all remaining args into variables.
# Stop parsing when all options are done. Try to do a check on all options,
# parameters and version numbers. Exit on error.

if [ "$debug" == "yes" ]; then				# DEBUG
	echo "DEBUG Parse Commandline: $# $@" >&2
	$debuglog Parse Commandline: $# $@
fi

# get command (1. cmdline parameter)
case "$1" in
    -h|--help)	cmd='-h'; shift;;			# cmd -h
    -V|--version)	cmd='-V'; shift;;		# cmd -V
#    -s)	cmd="$1"; shift;;			# cmd found
#    *)	$errlog unknown command $1 !
#	exit 3;;
esac

# get options
while [ "${1:0:1}" == "-" ]; do				# get all options
	case $1 in
	    -d|--debug)		debug=yes;;		# option -d
	    -q|--quiet)		verbose=no;;		# option -q
	    -v|--verbose)	verbose=yes;;		# option -v
#	    -a)	opt_a=$1;;				# option -a
	    *)	$errlog unknown option $1 !
		exit 3;;
	esac
	shift
done

# check version of config files
#if [ "$version" != "$Counter_Conf" ] || [ "$version" != "$Counter_Rc" ]; then
#	if [ "$verbose" != no ]; then
#		echo "Error in version of config files!" >&2
#	else
#		$errlog version of config files differ
#	fi
#	exit 1
#fi

# check arguments after commands and options
case "$cmd" in
    '')
	if [ "$#" -eq 1 ]; then				# parameter count ok
		arg1="$1"
		shift
	elif [ "$#" -eq 4 ]; then			# parameter count ok
		arg1="$1"
		arg2="$2"
		arg3="$3"
		arg4="$4"
		shift 4
	else						# error in param count
		if [ "$verbose" != no ]; then
			echo "Error in parameter count!" >&2
		else
			$errlog parameter count
		fi
		exit 2
	fi
	;;
    *)
	if [ "$#" -ne 0 ]; then				# error in param count
		if [ "$verbose" != no ]; then
			echo "Error in parameter count!" >&2
		else
			$errlog parameter count
		fi
		exit 2
	fi
	;;
esac

##############################################################################
# MAINPROGRAM

if [ "$debug" == "yes" ]; then				# DEBUG
	#echo "DEBUG Main: $# $@" >&2
	$debuglog Main: $# $@
fi

# Check what to do (initial cmds, options and args are removed from cmdline)
case "$cmd" in
    '')
	# setup data file, img file, nr
	datfn="$datadir/$arg1.dat"			# cntr data file
	imgfn="$cntrdir/$arg1.png"			# cntr img file
	nr=$(cat $datfn)				# get counter val

	# look for cgi cmd
	case "$arg2" in
	    "")	
		make_inc_image
		;;
	    set)
		set_counter
		err=$?
		if [ "$err" -eq 0 ]; then
			while [ "${#nr}" -lt 6 ]; do	# fillup leading 0
				nr=0$nr
			done

			make_image
		fi
		;;
	    *)
		if [ "$verbose" != no ]; then
			echo "Unknown usercmd!" >&2
		else
			$errlog unknown usercmd!
		fi
		exit 3
		;;
	esac
	;;
    -h)							# cmd -h
	helptext
	err=0
	;;
    -V)							# cmd -V
	echo "$scriptname (${package}) ${version}-${release}"
	err=0
	;;
    *)
	$errlog unknown command!
	exit 3
	;;
esac
